@@ -1,77 +1,8 @@
 # -*- coding: utf-8 -*-
+from abc import ABC, abstractmethod, abstractproperty
 from datetime import datetime, timedelta
 from .instance import AbstractBlockchainInstanceProvider
-
-
-class ObjectCache(dict):
-    """ This class implements an object/dict cache that comes with an
-        expiration. Expired items are removed from the cache.
-
-        The class implements/extends:
-
-         * __setitem__()
-         * __getitem__()
-         * __contains__
-         * __str__()
-         * get()
-
-        and provides a method to define the default expiration time.
-
-    """
-
-    def __init__(self, initial_data={}, default_expiration=60, no_overwrite=False):
-        dict.__init__(self, initial_data)
-
-        # Expiration
-        self.set_expiration(default_expiration)
-
-        # This allows nicer testing
-        self.no_overwrite = no_overwrite
-
-    def __setitem__(self, key, value):
-        if key in self and not self.no_overwrite:
-            del self[key]
-        elif key in self and self.no_overwrite:
-            return
-        data = {
-            "expires": datetime.utcnow() + timedelta(seconds=self.default_expiration),
-            "data": value,
-        }
-        dict.__setitem__(self, key, data)
-
-    def __getitem__(self, key):
-        if key in self:
-            value = dict.__getitem__(self, key)
-            return value["data"]
-
-    def get(self, key, default=None):
-        """ Returns an element from the cache if available, else returns
-            the value provided as default or None
-        """
-        if key in self:
-            return self[key]
-        else:
-            return default
-
-    def __contains__(self, key):
-        if dict.__contains__(self, key):
-            value = dict.__getitem__(self, key)
-            if datetime.utcnow() < value["expires"]:
-                return True
-            else:
-                # Remove from cache
-                dict.pop(self, key, None)
-                return False
-
-    def __str__(self):
-        return "ObjectCache(n={}, default_expiration={})".format(
-            len(self.keys()), self.default_expiration
-        )
-
-    def set_expiration(self, expiration):
-        """ Set new default expiration time in seconds (default: 10s)
-        """
-        self.default_expiration = expiration
+from .objectcache import ObjectCacheInMemory
 
 
 class Caching:
@@ -79,6 +10,18 @@ class Caching:
         either cache lists or dicts
     """
 
+    __caching_args = set()
+    __caching_kwargs = dict()
+    __caching_klass = ObjectCacheInMemory
+    _cache = ObjectCacheInMemory()
+
+    @staticmethod
+    def set_cache_store(klass, *args, **kwargs):
+        Caching.__caching_args = args
+        Caching.__caching_kwargs = kwargs
+        Caching.__caching_klass = klass
+        Caching._cache = klass(*args, **kwargs)
+
     def __init__(self, *args, **kwargs):
         self._fetched = False
 
@@ -132,7 +75,7 @@ class Caching:
     def clear_cache(cls):
         """ Clear/Reset the entire Cache
         """
-        cls._cache = ObjectCache()
+        cls._cache = ObjectCacheInMemory()
 
     __str__ = __repr__
 
@@ -142,7 +85,6 @@ class BlockchainObjects(Caching, list):
         deal with the cache and indexing thereof.
     """
 
-    _cache = ObjectCache()
     identifier = None
 
     def refresh(self, *args, **kwargs):
@@ -165,6 +107,8 @@ class BlockchainObjects(Caching, list):
         else:
             if kwargs.get("refresh", True):
                 self.refresh(*args, **kwargs)
+        if args:
+            self.identifier = args[0]
 
     def _cache_key(self, key=""):
         if key:
@@ -221,8 +165,6 @@ class BlockchainObject(Caching, dict):
     type_ids = []
     identifier = None
 
-    _cache = ObjectCache()
-
     def __init__(self, data, klass=None, lazy=False, use_cache=True, *args, **kwargs):
         Caching.__init__(self, *args, **kwargs)
         self._use_cache = use_cache
